@@ -1,9 +1,9 @@
 /*
- *  $Id: guage.c,v 1.64 2011/10/20 23:34:35 tom Exp $
+ *  $Id: guage.c,v 1.76 2018/06/21 08:23:43 tom Exp $
  *
  *  guage.c -- implements the gauge dialog
  *
- *  Copyright 2000-2010,2011	Thomas E. Dickey
+ *  Copyright 2000-2015,2018	Thomas E. Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License, version 2.1
@@ -39,7 +39,7 @@ typedef struct _my_obj {
     DIALOG_CALLBACK obj;	/* has to be first in struct */
     struct _my_obj *next;
     WINDOW *text;
-    const char *title;
+    char *title;
     char *prompt;
     char prompt_buf[MY_LEN];
     int percent;
@@ -94,6 +94,7 @@ read_data(char *buffer, FILE *fp)
 	result = 0;
     } else if (fgets(buffer, MY_LEN, fp) != 0) {
 	DLG_TRACE(("read_data:%s", buffer));
+	buffer[MY_LEN] = '\0';
 	dlg_trim_string(buffer);
 	result = 1;
     } else {
@@ -120,14 +121,14 @@ repaint_text(MY_OBJ * obj)
     WINDOW *dialog = obj->obj.win;
     int i, x;
 
-    if (dialog != 0 && obj->obj.input != 0) {
+    if (dialog != 0) {
 	(void) werase(dialog);
 	dlg_draw_box2(dialog, 0, 0, obj->height, obj->width, dialog_attr,
 		      border_attr, border2_attr);
 
 	dlg_draw_title(dialog, obj->title);
 
-	(void) wattrset(dialog, dialog_attr);
+	dlg_attrset(dialog, dialog_attr);
 	dlg_draw_helpline(dialog, FALSE);
 	dlg_print_autowrap(dialog, obj->prompt, obj->height, obj->width);
 
@@ -144,7 +145,7 @@ repaint_text(MY_OBJ * obj)
 	 * attribute.
 	 */
 	(void) wmove(dialog, obj->height - 3, 4);
-	(void) wattrset(dialog, gauge_attr);
+	dlg_attrset(dialog, gauge_attr);
 
 	for (i = 0; i < (obj->width - 2 * (3 + MARGIN)); i++)
 	    (void) waddch(dialog, ' ');
@@ -159,9 +160,9 @@ repaint_text(MY_OBJ * obj)
 	 */
 	x = (obj->percent * (obj->width - 2 * (3 + MARGIN))) / 100;
 	if ((gauge_attr & A_REVERSE) != 0) {
-	    wattroff(dialog, A_REVERSE);
+	    dlg_attroff(dialog, A_REVERSE);
 	} else {
-	    (void) wattrset(dialog, A_REVERSE);
+	    dlg_attrset(dialog, A_REVERSE);
 	}
 	(void) wmove(dialog, obj->height - 3, 4);
 	for (i = 0; i < x; i++) {
@@ -181,11 +182,13 @@ handle_input(DIALOG_CALLBACK * cb)
 {
     MY_OBJ *obj = (MY_OBJ *) cb;
     bool result;
+    bool cleanup = FALSE;
     int status;
-    char buf[MY_LEN];
+    char buf[MY_LEN + 1];
 
     if (dialog_state.pipe_input == 0) {
 	status = -1;
+	cleanup = TRUE;
     } else if ((status = read_data(buf, dialog_state.pipe_input)) > 0) {
 
 	if (isMarker(buf)) {
@@ -221,16 +224,19 @@ handle_input(DIALOG_CALLBACK * cb)
     } else {
 	if (feof(dialog_state.pipe_input) ||
 	    (ferror(dialog_state.pipe_input) && errno != EINTR)) {
-	    delink(obj);
-	    dlg_remove_callback(cb);
+	    cleanup = TRUE;
 	}
     }
 
+    repaint_text(obj);
     if (status > 0) {
 	result = TRUE;
-	repaint_text(obj);
     } else {
 	result = FALSE;
+	if (cleanup) {
+	    dlg_remove_callback(cb);
+	    delink(obj);
+	}
     }
 
     return result;
@@ -239,7 +245,7 @@ handle_input(DIALOG_CALLBACK * cb)
 static bool
 handle_my_getc(DIALOG_CALLBACK * cb, int ch, int fkey, int *result)
 {
-    int status = TRUE;
+    bool status = TRUE;
 
     *result = DLG_EXIT_OK;
     if (cb != 0) {
@@ -264,6 +270,8 @@ my_cleanup(DIALOG_CALLBACK * cb)
 	    free(obj->prompt);
 	    obj->prompt = obj->prompt_buf;
 	}
+	free(obj->title);
+	dlg_del_window(obj->obj.win);
 	delink(obj);
     }
 }
@@ -272,15 +280,87 @@ void
 dlg_update_gauge(void *objptr, int percent)
 {
     MY_OBJ *obj = (MY_OBJ *) objptr;
+    bool save_finish_string = dialog_state.finish_string;
 
+    dialog_state.finish_string = TRUE;
     curs_set(0);
     obj->percent = percent;
     repaint_text(obj);
+    dialog_state.finish_string = save_finish_string;
 }
 
 /*
- * Allocates a new object and fills it as per the arguments
+ * (Re)Allocates an object and fills it as per the arguments
  */
+void *
+dlg_reallocate_gauge(void *objptr,
+		     const char *title,
+		     const char *cprompt,
+		     int height,
+		     int width,
+		     int percent)
+{
+    char *prompt = dlg_strclone(cprompt);
+    MY_OBJ *obj = objptr;
+    bool save_finish_string = dialog_state.finish_string;
+
+    dialog_state.finish_string = TRUE;
+    dlg_tab_correct_str(prompt);
+
+    if (objptr == 0) {
+	/* create a new object */
+	obj = dlg_calloc(MY_OBJ, 1);
+	assert_ptr(obj, "dialog_gauge");
+
+	dlg_auto_size(title, prompt, &height, &width, MIN_HIGH, MIN_WIDE);
+	dlg_print_size(height, width);
+	dlg_ctl_size(height, width);
+
+    } else {
+	/* reuse an existing object */
+	obj = objptr;
+	height = obj->height;
+	width = obj->width;
+    }
+
+    if (obj->obj.win == 0) {
+	/* center dialog box on screen */
+	int x = dlg_box_x_ordinate(width);
+	int y = dlg_box_y_ordinate(height);
+	WINDOW *dialog = dlg_new_window(height, width, y, x);
+	obj->obj.win = dialog;
+    }
+
+    obj->obj.input = dialog_state.pipe_input;
+    obj->obj.keep_win = TRUE;
+    obj->obj.bg_task = TRUE;
+    obj->obj.handle_getc = handle_my_getc;
+    obj->obj.handle_input = handle_input;
+
+    if (obj->title == 0 || strcmp(obj->title, title)) {
+	dlg_finish_string(obj->title);
+	free(obj->title);
+	obj->title = dlg_strclone(title);
+    }
+
+    dlg_finish_string(obj->prompt);
+    free(obj->prompt);
+
+    obj->prompt = prompt;
+    obj->percent = percent;
+    obj->height = height;
+    obj->width = width;
+
+    /* if this was a new object, link it into the list */
+    if (objptr == 0) {
+	obj->next = all_objects;
+	all_objects = obj;
+    }
+
+    dialog_state.finish_string = save_finish_string;
+    return (void *) obj;
+}
+
 void *
 dlg_allocate_gauge(const char *title,
 		   const char *cprompt,
@@ -288,43 +368,7 @@ dlg_allocate_gauge(const char *title,
 		   int width,
 		   int percent)
 {
-    int x, y;
-    char *prompt = dlg_strclone(cprompt);
-    WINDOW *dialog;
-    MY_OBJ *obj = 0;
-
-    dlg_tab_correct_str(prompt);
-
-    dlg_auto_size(title, prompt, &height, &width, MIN_HIGH, MIN_WIDE);
-    dlg_print_size(height, width);
-    dlg_ctl_size(height, width);
-
-    /* center dialog box on screen */
-    x = dlg_box_x_ordinate(width);
-    y = dlg_box_y_ordinate(height);
-
-    dialog = dlg_new_window(height, width, y, x);
-
-    obj = dlg_calloc(MY_OBJ, 1);
-    assert_ptr(obj, "dialog_gauge");
-
-    obj->obj.input = dialog_state.pipe_input;
-    obj->obj.win = dialog;
-    obj->obj.keep_win = TRUE;
-    obj->obj.bg_task = TRUE;
-    obj->obj.handle_getc = handle_my_getc;
-    obj->obj.handle_input = handle_input;
-
-    obj->title = title;
-    obj->prompt = prompt;
-    obj->percent = percent;
-    obj->height = height;
-    obj->width = width;
-
-    obj->next = all_objects;
-    all_objects = obj;
-
-    return (void *) obj;
+    return dlg_reallocate_gauge(NULL, title, cprompt, height, width, percent);
 }
 
 void
@@ -332,12 +376,12 @@ dlg_free_gauge(void *objptr)
 {
     MY_OBJ *obj = (MY_OBJ *) objptr;
 
-    curs_set(1);
     if (valid(obj)) {
-	delink(obj);
 	obj->obj.keep_win = FALSE;
 	dlg_remove_callback(&(obj->obj));
+	delink(obj);
     }
+    curs_set(1);
 }
 
 /*
@@ -360,6 +404,13 @@ dialog_gauge(const char *title,
     void *objptr = dlg_allocate_gauge(title, cprompt, height, width, percent);
     MY_OBJ *obj = (MY_OBJ *) objptr;
 
+    DLG_TRACE(("# gauge args:\n"));
+    DLG_TRACE2S("title", title);
+    DLG_TRACE2S("message", cprompt);
+    DLG_TRACE2N("height", height);
+    DLG_TRACE2N("width", width);
+    DLG_TRACE2N("percent", percent);
+
     dlg_add_callback_ref((DIALOG_CALLBACK **) & obj, my_cleanup);
     dlg_update_gauge(obj, percent);
 
@@ -370,6 +421,7 @@ dialog_gauge(const char *title,
 	if (fkey && ch == KEY_RESIZE) {
 	    MY_OBJ *oldobj = obj;
 
+	    dlg_will_resize(obj->obj.win);
 	    dlg_mouse_free_regions();
 
 	    obj = dlg_allocate_gauge(title,
